// This file is licensed under the Elastic License 2.0. Copyright 2021 StarRocks Limited.

#pragma once

#include "common/object_pool.h"
#include "exprs/expr.h"

namespace starrocks {
namespace vectorized {

class ColumnRef final : public Expr {
public:
    ColumnRef(const TExprNode& node);

    ColumnRef(const SlotDescriptor* desc);

    SlotId slot_id() const { return _column_id; }

    TupleId tuple_id() const { return _tuple_id; }

    void set_slot_id(SlotId slot_id) { _column_id = slot_id; }

    void set_tuple_id(TupleId tuple_id) { _tuple_id = tuple_id; }

    // FixMe(kks): currenly, join runtime filter need this method
    // we should find a way remove this method
    bool is_bound(const std::vector<TupleId>& tuple_ids) const;

    Expr* clone(ObjectPool* pool) const override { return pool->add(new ColumnRef(*this)); }

    bool is_constant() const override { return false; }
    bool is_vectorized() const override { return true; }

    int get_slot_ids(std::vector<SlotId>* slot_ids) const override;

    std::string debug_string() const override;

    // vector query engine
    vectorized::ColumnPtr evaluate(ExprContext* context, vectorized::Chunk* ptr) override;

    static vectorized::ColumnPtr& get_column(Expr* expr, vectorized::Chunk* chunk);

private:
    // FixMe(kks): currenly, join runtime filter depend on _tuple_id.
    // we should find a way remove _tuple_id from ColumnRef
    SlotId _column_id;

    TupleId _tuple_id = 0; // used for desc this slot from
    bool _is_nullable = false;
};

inline vectorized::ColumnPtr& ColumnRef::get_column(Expr* expr, vectorized::Chunk* chunk) {
    ColumnRef* ref = (ColumnRef*)expr;
    ColumnPtr& column = (chunk)->get_column_by_slot_id(ref->slot_id());
    return column;
}

} // namespace vectorized
} // namespace starrocks
